package ap.proof;

import ap.proof.goal.{PrioritisedTask, Goal, TaskManager}
import ap.terfor.{Formula, ConstantTerm, TermOrder}
import ap.terfor.conjunctions.{Conjunction, NegatedConjunctions}
import ap.terfor.substitutions.Substitution
import ap.parameters.GoalSettings

/**
 * Proof tree factory in which updating a goal will recursive apply further
 * rules, until some stopping condition holds
 */
class IteratingProofTreeFactory(stoppingCond : (Goal) => Boolean,
                                simplifier : ConstraintSimplifier)
      extends SimpleProofTreeFactory(true, simplifier) {

  protected override def createNewGoal(updatedFacts : Conjunction,
                                       updatedClauses : NegatedConjunctions,
                                       updatedElimConstants : Set[ConstantTerm],
                                       updatedVocabulary : Vocabulary,
                                       updatedDefinedSyms : Substitution,
                                       newTasks : TaskManager,
                                       age : Int,
                                       settings : GoalSettings) : ProofTree = {
    val newGoal = Goal(updatedFacts, updatedClauses, newTasks, age,
                       updatedElimConstants,
                       updatedVocabulary, updatedDefinedSyms, settings)
    if (!newGoal.stepPossible || stoppingCond(newGoal))
      newGoal
    else
      newGoal step this
  }
  
}
