package ap.proof;

import ap.proof.goal.{PrioritisedTask, Goal, TaskManager}
import ap.terfor.{Formula, ConstantTerm, TermOrder}
import ap.terfor.conjunctions.{Conjunction, NegatedConjunctions, Quantifier}
import ap.terfor.substitutions.Substitution
import ap.parameters.GoalSettings

class SimpleProofTreeFactory(removeTask : Boolean,
                             simplifier : ConstraintSimplifier) extends ProofTreeFactory {

  def and(subtrees : Seq[ProofTree], vocabulary : Vocabulary) : ProofTree =
    AndTree(subtrees, vocabulary, simplifier)
  
  def quantify(subtree : ProofTree,
               quan : Quantifier,
               quantifiedConstants : Set[ConstantTerm],
               vocabulary : Vocabulary,
               subtreeOrder : TermOrder) : ProofTree =
    QuantifiedTree(subtree, quan, quantifiedConstants,
                   vocabulary, subtreeOrder, simplifier)
  
  def weaken(subtree : ProofTree,
             disjunct : Conjunction,
             vocabulary : Vocabulary) : ProofTree =
    WeakenTree(subtree, disjunct, vocabulary)
               
  // not supposed to do anything when constructing proofs
  def eliminatedConstant(subtree : ProofTree,
                         c : ConstantTerm,
                         witness : (Substitution, TermOrder) => Substitution,
                         vocabulary : Vocabulary) : ProofTree =
     subtree
    
  //////////////////////////////////////////////////////////////////////////////
  
  protected def createNewGoal(updatedFacts : Conjunction,
                              updatedClauses : NegatedConjunctions,
                              updatedElimConstants : Set[ConstantTerm],
                              updatedVocabulary : Vocabulary,
                              updatedDefinedSyms : Substitution,
                              newTasks : TaskManager,
                              age : Int,
                              settings : GoalSettings) : ProofTree =
    Goal(updatedFacts, updatedClauses, newTasks, age,
         updatedElimConstants, updatedVocabulary, updatedDefinedSyms, settings)
    
  def updateGoal(updatedFacts : Conjunction,
                 updatedClauses : NegatedConjunctions,
                 updatedElimConstants : Set[ConstantTerm],
                 updatedVocabulary : Vocabulary,
                 updatedDefinedSyms : Substitution,
                 newTasks : Iterable[PrioritisedTask],
                 goal : Goal) : ProofTree = {
    val updatedTasks = (if (removeTask)
                          goal.tasks.removeFirst
                        else
                          goal.tasks) ++ newTasks
    createNewGoal(updatedFacts, updatedClauses, updatedElimConstants,
                  updatedVocabulary, updatedDefinedSyms,
                  updatedTasks, goal.age + 1, goal.settings)
  }
  
  def updateGoal(tasks : TaskManager, goal : Goal) : ProofTree =
    createNewGoal(goal.facts, goal.clauses,
                  goal.eliminatedConstants, goal.vocabulary, goal.definedSyms,
                  tasks, goal.age + 1, goal.settings)

}
