package ap.terfor.substitutions;

import ap.basetypes.IdealInt
import ap.util.Debug

object PseudoConstantSubst {
  
  private val AC = Debug.AC_SUBSTITUTIONS
  
  def unapply(s : Substitution) : Option[(IdealInt, ConstantTerm, Term)] = s match {
    case s : PseudoConstantSubst => Some((s.coeff, s.constant, s.replacement))
    case _ => None
  }
  
}

/**
 * Replace a constant with an arbitrary term
 */
class PseudoConstantSubst(private val coeff : IdealInt,
                          private val constant : ConstantTerm,
                          private val replacement : Term,
                          protected [substitutions] val order : TermOrder)
      extends PseudoDivSubstitution {

  //////////////////////////////////////////////////////////////////////////////
  Debug.assertCtor(PseudoConstantSubst.AC,
                   (order isSortingOf replacement) && !coeff.isZero)
  //////////////////////////////////////////////////////////////////////////////     

  protected[substitutions] def passQuantifiers(num : Int) : Substitution =
    new PseudoConstantSubst(coeff,
                            constant,
                            // TODO: could be optimised by caching
                            new VariableShiftSubst(0, num, order) (replacement),
                            order)
    
  protected[substitutions] def applyToVariable(v : VariableTerm) : (IdealInt, Term) =
    (IdealInt.ONE, v)

  protected[substitutions] def applyToConstant(c : ConstantTerm) : (IdealInt, Term) =
    if (c == constant) (coeff, replacement) else (IdealInt.ONE, c)

  protected[substitutions] def isIdentityOn(t : TerFor) : Boolean =
    !(t.constants contains constant)

  //////////////////////////////////////////////////////////////////////////////

  def sortBy(newOrder : TermOrder) : PseudoConstantSubst =
    new PseudoConstantSubst(coeff, constant, newOrder sort replacement, newOrder)

  //////////////////////////////////////////////////////////////////////////////

  override def toString : String = "[" + coeff + "*" + constant + " |-> " + replacement + "]"

}
