package ap.terfor.substitutions;

import ap.util.{Debug, Logic, APTestCase, PlainRange}
import ap.basetypes.IdealInt
import ap.terfor.linearcombination.LinearCombination
import ap.terfor.equations.EquationConj

class TestSubst(n : String) extends APTestCase(n) {

  def runTest = {
    n match {
      case "testPseudoSubst" => testPseudoSubst
    }
  }

  private val consts = for (i <- Array.range(0, 10)) yield new ConstantTerm("c" + i)
  private val constsOne = consts ++ List(OneTerm)
  private val to = (TermOrder.EMPTY /: consts)((o, c) => o.extend(c, Set.empty))
  private val constsLC = for (t <- consts) yield LinearCombination(t, to)

  def testPseudoSubst = {
    val lc0 = constsLC(0) * 5 + constsLC(1) + 6
    val subst0 = new PseudoConstantSubst(3, consts(1), constsLC(2) * 2, to)
    
    assertEquals(subst0(EquationConj(lc0, to)),
                 EquationConj(constsLC(0) * 15 + constsLC(2) * 2 + 18, to))

    val subst1 = new PseudoConstantSubst(3, consts(1), constsLC(2) * 5, to)
                 
    assertEquals(subst1(EquationConj(lc0, to)),
                 EquationConj.FALSE)

    val lc1 = constsLC(0) * 5 + constsLC(1) * 3 + 6
    assertEquals(subst0(EquationConj(lc1, to)),
                 EquationConj(constsLC(0) * 5 + constsLC(2) * 2 + 6, to))
  }
  
}
